/** @file sys_startup.c 
*   @brief Startup Source File
*   @date 05.November.2010
*   @version 1.01.001
*
*   This file contains:
*   - Include Files
*   - Type Definitions
*   - External Functions
*   - VIM RAM Setup
*   - Startup Routine
*   .
*   which are relevant for the Starup.
*/

/* (c) Texas Instruments 2010, All rights reserved. */

/* USER CODE BEGIN (0) */
/* USER CODE END */


/* Include Files */

#include "sys_types.h"
#include "sys_common.h"
#include "system.h"
#include "sys_vim.h"
#include "sys_core.h"
#include "sys_memory.h"

/* USER CODE BEGIN (1) */
/* USER CODE END */


/* Type Definitions */
typedef void (*handler_fptr)(const unsigned char *in, unsigned char *out);

/* USER CODE BEGIN (2) */
/* USER CODE END */


/* External Functions */
#pragma WEAK(__TI_Handler_Table_Base)
#pragma WEAK(__TI_Handler_Table_Limit)
#pragma WEAK(__TI_CINIT_Base)
#pragma WEAK(__TI_CINIT_Limit)

extern unsigned   __TI_Handler_Table_Base;
extern unsigned   __TI_Handler_Table_Limit;
extern unsigned   __TI_CINIT_Base;
extern unsigned   __TI_CINIT_Limit;
extern unsigned   __TI_PINIT_Base;
extern unsigned   __TI_PINIT_Limit;
extern unsigned * __binit__;

extern void main(void);
extern void exit(void);

/* USER CODE BEGIN (3) */
/* USER CODE END */


/* Vim Ram Definition */
/** @struct vimRam
*   @brief Vim Ram Definition
*
*   This type is used to access the Vim Ram.
*/
/** @typedef vimRAM_t
*   @brief Vim Ram Type Definition
*
*   This type is used to access the Vim Ram.
*/
typedef volatile struct vimRam
{
    t_isrFuncPTR ISR[VIM_CHANNELS];
} vimRAM_t;

#define vimRAM ((vimRAM_t *)0xFFF82000U)

static const t_isrFuncPTR s_vim_init[] =
{
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
    phantomInterrupt,
};


/* Startup Routine */

/* USER CODE BEGIN (4) */
/* USER CODE END */

#pragma INTERRUPT(_c_int00, RESET)

void _c_int00()
{
    /* Enable VFP Unit */
    _coreEnableVfp_();

    /* Initialize Core Registers */
    _coreInitRegisters_();

    /* Initialize Stack Pointers */
    _coreInitStackPointer_();

    /* Enable IRQ offset via Vic controller */
    _coreEnableIrqVicOffset_();

    /* Initialize System */
    systemInit();


	/* device safety settings */
	_coreEnableEventBusExport_();
//	_coreEnableRamEcc_();

    /* Run memory tests */

    /* Initialize memory */
//  _memoryInit_();  /* not possible due to swapped Flash & RAM */

    /* Initialize VIM table */
    {
        unsigned i;

        for (i = 0; i < 64U; i++)
        {
            vimRAM->ISR[i] = s_vim_init[i];
        }
    }

    /* set IRQ/FIQ priorities */
    vimREG->FIRQPR0 =  SYS_FIQ
                    | (SYS_FIQ <<  1U)
                    | (SYS_IRQ <<  2U)
                    | (SYS_IRQ <<  3U)
                    | (SYS_IRQ <<  4U)
                    | (SYS_IRQ <<  5U)
                    | (SYS_IRQ <<  6U)
                    | (SYS_IRQ <<  7U)
                    | (SYS_IRQ <<  8U)
                    | (SYS_IRQ <<  9U)
                    | (SYS_IRQ << 10U)
                    | (SYS_IRQ << 11U)
                    | (SYS_IRQ << 12U)
                    | (SYS_IRQ << 13U)
                    | (SYS_IRQ << 14U)
                    | (SYS_IRQ << 15U)
                    | (SYS_IRQ << 16U)
                    | (SYS_IRQ << 17U)
                    | (SYS_IRQ << 18U)
                    | (SYS_IRQ << 19U)
                    | (SYS_IRQ << 20U)
                    | (SYS_IRQ << 21U)
                    | (SYS_IRQ << 22U)
                    | (SYS_IRQ << 23U)
                    | (SYS_IRQ << 24U)
                    | (SYS_IRQ << 25U)
                    | (SYS_IRQ << 26U)
                    | (SYS_IRQ << 27U)
                    | (SYS_IRQ << 28U)
                    | (SYS_IRQ << 29U)
                    | (SYS_IRQ << 30U)
                    | (SYS_IRQ << 31U);

    vimREG->FIRQPR1 =  SYS_IRQ
                    | (SYS_IRQ <<  1U)
                    | (SYS_IRQ <<  2U)
                    | (SYS_IRQ <<  3U)
                    | (SYS_IRQ <<  4U)
                    | (SYS_IRQ <<  5U)
                    | (SYS_IRQ <<  6U)
                    | (SYS_IRQ <<  7U)
                    | (SYS_IRQ <<  8U)
                    | (SYS_IRQ <<  9U)
                    | (SYS_IRQ << 10U)
                    | (SYS_IRQ << 11U)
                    | (SYS_IRQ << 12U)
                    | (SYS_IRQ << 13U)
                    | (SYS_IRQ << 14U)
                    | (SYS_IRQ << 15U)
                    | (SYS_IRQ << 16U)
                    | (SYS_IRQ << 17U)
                    | (SYS_IRQ << 18U)
                    | (SYS_IRQ << 19U)
                    | (SYS_IRQ << 20U)
                    | (SYS_IRQ << 21U)
                    | (SYS_IRQ << 22U)
                    | (SYS_IRQ << 23U)
                    | (SYS_IRQ << 24U)
                    | (SYS_IRQ << 25U)
                    | (SYS_IRQ << 26U)
                    | (SYS_IRQ << 27U)
                    | (SYS_IRQ << 28U)
                    | (SYS_IRQ << 29U)
                    | (SYS_IRQ << 30U);

    /* enable interrupts */
    vimREG->REQMASKSET0 = 1U
                        | (0U << 1)
                        | (0U << 2)
                        | (0U << 3)
                        | (0U << 4)
                        | (0U << 5)
                        | (0U << 6)
                        | (0U << 7)
                        | (0U << 8)
                        | (0U << 9)
                        | (0U << 10)
                        | (0U << 11)
                        | (0U << 12)
                        | (0U << 13)
                        | (0U << 14)
                        | (0U << 15)
                        | (0U << 16)
                        | (0U << 17)
                        | (0U << 18)
                        | (0U << 19)
                        | (0U << 20)
                        | (0U << 21)
                        | (0U << 22)
                        | (0U << 23)
                        | (0U << 24)
                        | (0U << 25)
                        | (0U << 26)
                        | (0U << 27)
                        | (0U << 28)
                        | (0U << 29)
                        | (0U << 30)
                        | (0U << 31);

    vimREG->REQMASKSET1 = 0U
                        | (0U << 1)
                        | (0U << 2)
                        | (0U << 3)
                        | (0U << 4)
                        | (0U << 5)
                        | (0U << 6)
                        | (0U << 7)
                        | (0U << 8)
                        | (0U << 9)
                        | (0U << 10)
                        | (0U << 11)
                        | (0U << 12)
                        | (0U << 13)
                        | (0U << 14)
                        | (0U << 15)
                        | (0U << 16)
                        | (0U << 17)
                        | (0U << 18)
                        | (0U << 19)
                        | (0U << 20)
                        | (0U << 21)
                        | (0U << 22)
                        | (0U << 23)
                        | (0U << 24)
                        | (0U << 25)
                        | (0U << 26)
                        | (0U << 27)
                        | (0U << 28)
                        | (0U << 29)
                        | (0U << 30);


    /* initalise copy table */
    if ((unsigned *)&__binit__ != (unsigned *)0xFFFFFFFF)
    {
        extern void copy_in(void *binit);
        copy_in((void *)&__binit__);
    }

    /* initalise the C global variables */
    if (&__TI_Handler_Table_Base < &__TI_Handler_Table_Limit)
    {
        unsigned char **tablePtr   = (unsigned char **)&__TI_CINIT_Base;
        unsigned char **tableLimit = (unsigned char **)&__TI_CINIT_Limit;

        while (tablePtr < tableLimit)
        {
            unsigned char *loadAdr = *tablePtr++;
            unsigned char *runAdr  = *tablePtr++;
            unsigned char  idx     = *loadAdr++;
            handler_fptr   handler = (handler_fptr)(&__TI_Handler_Table_Base)[idx];

            (*handler)((const unsigned char *)loadAdr, runAdr);
        }
    }

    /* initalise contructors */
    if (__TI_PINIT_Base < __TI_PINIT_Limit)
    {
        void (**p0)() = (void *)__TI_PINIT_Base;

        while ((unsigned)p0 < __TI_PINIT_Limit)
        {
            void (*p)() = *p0++;
            p();
        }
    }

/* USER CODE BEGIN (5) */
/* USER CODE END */

    /* call the application */
    main();
    exit();
}


/* USER CODE BEGIN (6) */
/* USER CODE END */
